{% extends "!layout.html" %}


{% block header %}
  {{ super() }}
  <a href="https://github.com/spacetelescope/asdf-standard"
     class="visible-desktop"><img
    style="position: absolute; top: 40px; right: 0; border: 0;"
    src="https://s3.amazonaws.com/github/ribbons/forkme_right_white_ffffff.png"
    alt="Fork me on GitHub"></a>
{% endblock %}
